﻿using System.Collections.Generic;
using Telerik.OpenAccess.Metadata.Fluent;
using Telerik.OpenAccess.Metadata;

namespace MvcForum.DomainObjects
{
    public class Forum : IEntity, INameable, ISlugable, IOrderable
    {
        private IList<Thread> threads = new List<Thread>();

        public virtual int Id { get; set; }

        public virtual string Name { get; set; }

        public virtual string Slug { get; set; }

        public virtual int Order { get; set; }

        public virtual IList<Thread> Threads { get { return threads; } }

        public virtual Category Category { get; set; }

        public virtual int CategoryId { get; set; }

        public static MappingConfiguration<Forum> CreateConfiguration()
        {
            var forumMapping = new MappingConfiguration<Forum>();
            forumMapping.MapType(f => new
            {
                Id = f.Id,
                Name = f.Name,
                Slug = f.Slug,
                Order = f.Order,
                CategoryId = f.CategoryId
            }).ToTable("Forum");
            forumMapping.HasProperty(f => f.Id).IsIdentity(KeyGenerator.Autoinc);
            forumMapping.HasAssociation(f => f.Threads).WithOpposite(t => t.Forum);
            forumMapping.HasAssociation(f => f.Category).WithOpposite(c => c.Forums).HasConstraint((f, c) => f.CategoryId == c.Id);

            return forumMapping;
        }
    }
}