﻿using System;
using Telerik.OpenAccess.Metadata.Fluent;
using Telerik.OpenAccess.Metadata;

namespace MvcForum.DomainObjects
{
    public class Post : IEntity, IOwnership
    {
        public virtual int Id { get; set; }

        public virtual string Content { get; set; }

        public virtual DateTime PostedAt { get; set; }

        public virtual User User { get; set; }

        public virtual int UserId { get; set; }

        public virtual Thread Thread { get; set; }

        public virtual int ThreadId { get; set; }

        public virtual bool IsPostedByOwner()
        {
            return (Thread != null) && (Thread.User != null) && IsOwner(Thread.User);
        }

        public virtual bool IsOwner(User user)
        {
            return (User != null) && (User.Id == user.Id);
        }

        public static MappingConfiguration<Post> CreateConfiguration()
        {
            var postMapping = new MappingConfiguration<Post>();
            postMapping.MapType(p => new
            {
                Id = p.Id,
                Content = p.Content,
                PostedAt = p.PostedAt,
                ThreadId = p.ThreadId,
                UserId = p.UserId
            }).ToTable("Post");
            postMapping.HasProperty(p => p.Id).IsIdentity(KeyGenerator.Autoinc);
            postMapping.HasAssociation(P => P.Thread).WithOpposite(t => t.Posts).HasConstraint((p, t) => p.ThreadId == t.Id);
            postMapping.HasAssociation(p => p.User).WithOpposite(u => u.Posts).HasConstraint((p, u) => p.UserId == u.Id);

            return postMapping;
        }
    }
}