﻿using System;
using System.Collections.Generic;
using Telerik.OpenAccess.Metadata.Fluent;
using Telerik.OpenAccess.Metadata;

namespace MvcForum.DomainObjects
{    
    public class Thread : IEntity, ISlugable, IOwnership
    {
        private IList<Post> posts = new List<Post>();

        public virtual int Id { get; set; }

        public virtual string Title { get; set; }

        public virtual string Slug { get; set; }

        public virtual User User { get; set; }

        public virtual int UserId { get; set; }

        public virtual Forum Forum { get; set; }

        public virtual int ForumId { get; set; }

        public virtual DateTime CreatedAt { get; set; }

        public virtual IList<Post> Posts { get { return posts; } }

        public virtual bool IsOwner(User user)
        {
            return (User != null) && (User.Id == user.Id);
        }

        public static MappingConfiguration<Thread> CreateConfiguration()
        {
            var threadMapping = new MappingConfiguration<Thread>();
            threadMapping.MapType(t => new
            {
                Id = t.Id,
                Title = t.Title,
                Slug = t.Slug,
                CreatedAt = t.CreatedAt,
                ForumId = t.ForumId,
                UserId = t.UserId
            }).ToTable("Thread");
            threadMapping.HasProperty(t => t.Id).IsIdentity(KeyGenerator.Autoinc);
            threadMapping.HasAssociation(t => t.Posts).WithOpposite(p => p.Thread);
            threadMapping.HasAssociation(t => t.Forum).WithOpposite(f => f.Threads).HasConstraint((t, f) => t.ForumId == f.Id);
            threadMapping.HasAssociation(t => t.User).WithOpposite(u => u.Threads).HasConstraint((t, u) => t.UserId == u.Id);

            return threadMapping;
        }
    }
}