﻿using System.Collections.Generic;
using Telerik.OpenAccess.Metadata;
using Telerik.OpenAccess.Metadata.Fluent;

namespace MvcForum.DomainObjects
{
    public class User : IEntity, INameable
    {
        private IList<Thread> threads = new List<Thread>();
        private IList<Post> posts = new List<Post>();

        public virtual int Id { get; set; }

        public virtual string Name { get; set; }

        public virtual string Password { get; set; }

        public virtual string Email { get; set; }

        public virtual bool IsLockedOut { get; set; }

        public virtual IList<Thread> Threads { get { return threads; } }

        public virtual IList<Post> Posts { get { return posts; } }

        public static MappingConfiguration<User> CreateConfiguration()
        {
            var userMapping = new MappingConfiguration<User>();
            userMapping.MapType(u => new
            {
                Id = u.Id,
                Name = u.Name,
                Password = u.Password,
                Email = u.Email,
                IsLockedOut = u.IsLockedOut
            }).ToTable("User");
            userMapping.HasProperty(u => u.Id).IsIdentity(KeyGenerator.Autoinc);
            userMapping.HasAssociation(u => u.Threads).WithOpposite(t => t.User);
            userMapping.HasAssociation(u => u.Posts).WithOpposite(p => p.User);

            return userMapping;
        }
    }
}