﻿using System.Globalization;
using System.Linq;

namespace MvcForum
{
    public static class StringExtensions
    {
        private static readonly char[] IllegalUrlCharacters = new[] { ';', '/', '\\', '?', ':', '@', '&', '=', '+', '$', ',', '<', '>', '#', '%', '.', '!', '*', '\'', '"', '(', ')', '[', ']', '{', '}', '|', '^', '`', '~', '–', '‘', '’', '“', '”', '»', '«' };

        public static string ToUrlSafe(this string value)
        {
            if (value.IsNullOrWhiteSpace())
            {
                return value;
            }

            value = value.Trim();

            if (value.IndexOfAny(IllegalUrlCharacters) > -1)
            {
                value = IllegalUrlCharacters.Aggregate(value, (current, character) => current.Replace(character.ToString(CultureInfo.CurrentCulture), string.Empty));
            }

            value = value.Replace(" ", "-");

            while (value.Contains("--"))
            {
                value = value.Replace("--", "-");
            }

            return value;
        }

        public static bool IsNullOrWhiteSpace(this string value)
        {
            return string.IsNullOrEmpty(value) || value.Trim().Length == 0;
        }
    }
}