﻿using System;
using Telerik.OpenAccess;
using MvcForum.Core;

namespace MvcForum.Infrastructure.OpenAccess
{
    public class ContextFactory : IContextFactory
    {
        private bool isDisposed;

        private readonly string connectionString;

        private OpenAccessContext context;

        public ContextFactory(string connectionString)
        {
            this.connectionString = connectionString;
        }

        public OpenAccessContext Get()
        {
            if (context == null)
            {
                var metadata = new MvcForumFluentMetadataSource();

                context = new OpenAccessContext(connectionString, new BackendConfiguration() { Backend = "mssql" }, metadata);
            }

            return context;
        }

        ~ContextFactory()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!isDisposed && disposing && (context != null))
                context.Dispose();

            isDisposed = true;
        }
    }
}