﻿using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using System.Web;
using MvcForum.ViewModels;
using MvcForum.Service;
namespace MvcForum.Web
{
    public class ThreadController : Controller
    {
        private readonly IForumService forumService;

        public ThreadController(IForumService forumService)
        {
            this.forumService = forumService;
        }

        public ActionResult Index(CategoryViewModel category, ForumViewModel forum, int? page, string orderBy, string filter)
        {
            if ((category == null) || (forum == null) || (category.Id != forum.CategoryId))
            {
                return new NotFoundResult();
            }

            IEnumerable<ThreadViewModel> threads = forumService.GetThreadsByForumId(forum.Id);

            if (orderBy.IsNullOrWhiteSpace())
            {
                threads = threads.OrderByDescending(t => t.CreatedAt);
            }

            ThreadListViewModel model = new ThreadListViewModel(threads) { Category = category, Forum = forum };

            return View(model);
        }

        public ActionResult Details(CategoryViewModel category, ForumViewModel forum, ThreadViewModel thread, int? page, string orderBy, string filter)
        {
            if ((category == null) || (forum == null) || (thread == null) || (category.Id != forum.CategoryId) || (forum.Id != thread.ForumId))
            {
                return new NotFoundResult();
            }

            IEnumerable<PostViewModel> posts = forumService.GetPostsByThreadId(thread.Id);

            if (orderBy.IsNullOrWhiteSpace())
            {
                posts = posts.OrderBy(p => p.PostedAt);
            }

            PostListViewModel model = new PostListViewModel(posts) { Category = category, Forum = forum, Thread = thread };

            return View(model);
        }

        [Authorize]
        public ActionResult Create(CategoryViewModel category, ForumViewModel forum)
        {
            return View(new CreateThreadForm { Category = category, Forum = forum });
        }

        [HttpPost, Authorize, ValidateInput(false)]
        public ActionResult Create(CreateThreadForm form, UserViewModel user)
        {
            if (ModelState.IsValid)
            {
                forumService.CreateThread(form.Title, HttpUtility.HtmlDecode(form.Content), form.Forum.Id, user.Id);

                return RedirectToAction("Index", new { category = form.Forum.CategorySlug, forum = form.Forum.Slug });
            }

            return View(form);
        }

        [Authorize]
        public ActionResult Reply(CategoryViewModel category, ForumViewModel forum, PostViewModel post)
        {
            return View(new ReplyThreadForm { Category = category, Forum = forum, Post = post });
        }

        [HttpPost, Authorize, ValidateInput(false)]
        public ActionResult Reply(ReplyThreadForm form, UserViewModel user)
        {
            if (ModelState.IsValid)
            {
                forumService.ReplyThread(HttpUtility.HtmlDecode(form.Content), form.Post.ThreadId, user.Id);

                return RedirectToAction("Details", new { category = form.Forum.CategorySlug, forum = form.Forum.Slug, thread = form.Post.ThreadId });
            }

            return View(form);
        }
    }
}