﻿using System.Globalization;
using System.Security.Cryptography;
using System.Text;
using MvcForum.ViewModels;

namespace MvcForum.Web
{
    public static class GravatarExtensions
    {
        private static int defaultSize = 80;

        public static int DefaultSize
        {
            get { return defaultSize; }
            set { defaultSize = value; }
        }

        public static string Gravatar(this UserViewModel instance)
        {
            return Gravatar(instance, DefaultSize);
        }

        public static string Gravatar(this UserViewModel instance, int size)
        {
            return GenerateUrl(((instance == null) ? string.Empty : instance.Email), size);
        }

        public static string Gravatar(this ThreadViewModel instance)
        {
            return Gravatar(instance, DefaultSize);
        }

        public static string Gravatar(this ThreadViewModel instance, int size)
        {
            return GenerateUrl(((instance == null) ? string.Empty : instance.UserEmail), size);
        }

        public static string Gravatar(this PostViewModel instance)
        {
            return Gravatar(instance, DefaultSize);
        }

        public static string Gravatar(this PostViewModel instance, int size)
        {
            return GenerateUrl(((instance == null) ? string.Empty : instance.UserEmail), size);
        }

        private static string GenerateUrl(string email, int size)
        {
            const string UrlFormat = "http://www.gravatar.com/avatar/{0}?r=G&s={1}&d=wavatar";

            StringBuilder result = new StringBuilder();
            CultureInfo culture = CultureInfo.InvariantCulture;

            if (!email.IsNullOrWhiteSpace())
            {
                byte[] hash;

                using (MD5 md5 = MD5.Create())
                {
                    byte[] data = Encoding.Default.GetBytes(email.ToLower(culture));

                    hash = md5.ComputeHash(data);
                }

                foreach (byte t in hash)
                {
                    result.Append(t.ToString("x2", culture));
                }
            }

            return string.Format(culture, UrlFormat, result, size);
        }
    }
}