﻿using System.Linq;
using Kendo.Mvc;
using MvcForum.ViewModels;
using MvcForum.Service;
using MvcForum.Web;

namespace MvcForum.Infrastructure
{
    public class ForumSiteMap : SiteMapBase
    {
        public ForumSiteMap(IForumService forumService)
        {
            Load(forumService);
        }

        private void Load(IForumService forumService)
        {
            RootNode.Title = "Home";

            var mapBuilder = new SiteMapBuilder(this);

            mapBuilder.RootNode.ChildNodes(node => node.Add().Action<HomeController>(c => c.Index(null, null, null)).Title("All Forums"));

            var rootBuilder = new SiteMapNodeBuilder(RootNode.ChildNodes[0]);

            foreach (CategoryViewModel category in forumService.GetCategories().OrderBy(c => c.Order))
            {
                CategoryViewModel localCategory = category;

                rootBuilder.ChildNodes(categoryNode =>
                {
                    var nodeBuilder = categoryNode.Add()
                                                  .Title(localCategory.Name)
                                                  .Action<ForumController>(c => c.Index(localCategory, null, null, null));

                    foreach (ForumViewModel forum in forumService.GetForumsByCategoryId(localCategory.Id).OrderBy(f => f.Order))
                    {
                        ForumViewModel localForumn = forum;

                        nodeBuilder.ChildNodes(forumNode => forumNode.Add()
                                                                     .Title(localForumn.Name)
                                                                     .Action<ThreadController>(c => c.Index(localCategory, localForumn, null, null, null)));
                    }
                });
            }
        }
    }
}