namespace MvcForum
{
    using System;

    using DomainObjects;

    public static class TestData
    {
        public static User CreateUser1()
        {
            return new User { Name = Random(), Password = Random(), Email = Random(), IsLockedOut = false };
        }

        public static User CreateUser2()
        {
            return new User { Id = 2, Name = Random(), Password = Random(), Email = Random(), IsLockedOut = false };
        }

        public static Category CreateCategory1()
        {
            return new Category { Name = Random(), Slug = Random(), Order = 1 };
        }

        public static Category CreateCategory2()
        {
            return new Category { Id = 2, Name = Random(), Slug = Random(), Order = 2 };
        }

        public static Forum CreateForum1()
        {
            return CreateForum1(false);
        }

        public static Forum CreateForum1(bool withParent)
        {
            var forum = new Forum { Name = Random(), Slug = Random(), Order = 1 };

            if (withParent)
            {
                forum.Category = CreateCategory1();
            }

            return forum;
        }

        public static Forum CreateForum2()
        {
            return CreateForum2(false);
        }

        public static Forum CreateForum2(bool withParent)
        {
            var forum = new Forum { Id = 2, Name = Random(), Slug = Random(), Order = 2 };

            if (withParent)
            {
                forum.Category = CreateCategory2();
            }

            return forum;
        }

        public static Thread CreateThread1()
        {
            return CreateThread1(false);
        }

        public static Thread CreateThread1(bool withParent)
        {
            var thread = new Thread { Title = Random(), Slug = Random(), CreatedAt = SystemTime.Now() };

            if (withParent)
            {
                thread.Forum = CreateForum1(true);
                thread.User = CreateUser1();
            }

            return thread;
        }

        public static Thread CreateThread2()
        {
            return CreateThread2(false);
        }

        public static Thread CreateThread2(bool withParent)
        {
            var thread = new Thread { Id = 2, Title = Random(), Slug = Random(), CreatedAt = SystemTime.Now() };

            if (withParent)
            {
                thread.Forum = CreateForum2(true);
                thread.User = CreateUser2();
            }

            return thread;
        }

        public static Post CreatePost1()
        {
            return CreatePost1(false);
        }

        public static Post CreatePost1(bool withParent)
        {
            var post = new Post { Content = Random(), PostedAt = SystemTime.Now() };

            if (withParent)
            {
                post.Thread = CreateThread1(true);
                post.User = CreateUser1();
            }

            return post;
        }

        public static Post CreatePost2()
        {
            return CreatePost2(false);
        }

        public static Post CreatePost2(bool withParent)
        {
            var post = new Post {Id = 2, Content = Random(), PostedAt = SystemTime.Now()};

            if (withParent)
            {
                post.Thread = CreateThread2(true);
                post.User = CreateUser2();
            }

            return post;
        }

        private static string Random()
        {
            return Guid.NewGuid().ToString();
        }
    }
}